@@ -16,8 +16,8 @@
 //
 //
 
-#ifndef GRPC_CORE_EXT_TRANSPORT_CHTTP2_ALPN_ALPN_H
-#define GRPC_CORE_EXT_TRANSPORT_CHTTP2_ALPN_ALPN_H
+#ifndef GRPC_SRC_CORE_EXT_TRANSPORT_CHTTP2_ALPN_ALPN_H
+#define GRPC_SRC_CORE_EXT_TRANSPORT_CHTTP2_ALPN_ALPN_H
 
 #include <grpc/support/port_platform.h>
 
@@ -33,4 +33,4 @@ size_t grpc_chttp2_num_alpn_versions(void);
 // grpc_chttp2_num_alpn_versions())
 const char* grpc_chttp2_get_alpn_version_index(size_t i);
 
-#endif  // GRPC_CORE_EXT_TRANSPORT_CHTTP2_ALPN_ALPN_H
+#endif  // GRPC_SRC_CORE_EXT_TRANSPORT_CHTTP2_ALPN_ALPN_H
