@@ -30,6 +30,7 @@
 
 #include "absl/functional/function_ref.h"
 #include "absl/status/status.h"
+#include "absl/strings/string_view.h"
 #include "absl/types/optional.h"
 
 #include <grpc/support/log.h>
@@ -39,7 +40,6 @@
 #include "src/core/lib/gprpp/time.h"
 #include "src/core/lib/resource_quota/memory_quota.h"
 #include "src/core/lib/transport/bdp_estimator.h"
-#include "src/core/lib/transport/pid_controller.h"
 
 extern grpc_core::TraceFlag grpc_flowctl_trace;
 
@@ -168,7 +168,7 @@ std::ostream& operator<<(std::ostream& out, const FlowControlAction& action);
 // to be as performant as possible.
 class TransportFlowControl final {
  public:
-  explicit TransportFlowControl(const char* name, bool enable_bdp_probe,
+  explicit TransportFlowControl(absl::string_view name, bool enable_bdp_probe,
                                 MemoryOwner* memory_owner);
   ~TransportFlowControl() {}
 
@@ -178,7 +178,17 @@ class TransportFlowControl final {
   // else returns zero; writing_anyway indicates if a write would happen
   // regardless of the send - if it is false and this function returns non-zero,
   // this announce will cause a write to occur
-  uint32_t MaybeSendUpdate(bool writing_anyway);
+  uint32_t DesiredAnnounceSize(bool writing_anyway) const;
+  // notify that we've actually sent a stream window update
+  // (should be DesiredAnnounceSize())
+  void SentUpdate(uint32_t announce);
+
+  // Older API: combines getting the DesiredAnnounceSize() with SentUpdate()
+  uint32_t MaybeSendUpdate(bool writing_anyway) {
+    uint32_t n = DesiredAnnounceSize(writing_anyway);
+    SentUpdate(n);
+    return n;
+  }
 
   // Track an update to the incoming flow control counters - that is how many
   // tokens we report to our peer that we're willing to accept.
@@ -259,7 +269,10 @@ class TransportFlowControl final {
   BdpEstimator* bdp_estimator() { return &bdp_estimator_; }
 
   uint32_t acked_init_window() const { return acked_init_window_; }
-  uint32_t sent_init_window() const { return target_initial_window_size_; }
+  uint32_t queued_init_window() const { return target_initial_window_size_; }
+  uint32_t sent_init_window() const { return sent_init_window_; }
+
+  void FlushedSettings() { sent_init_window_ = queued_init_window(); }
 
   FlowControlAction SetAckedInitialWindow(uint32_t value);
 
@@ -277,11 +290,47 @@ class TransportFlowControl final {
     }
   }
 
+  // A snapshot of the flow control stats to export.
+  struct Stats {
+    int64_t target_window;
+    int64_t target_frame_size;
+    int64_t target_preferred_rx_crypto_frame_size;
+    uint32_t acked_init_window;
+    uint32_t queued_init_window;
+    uint32_t sent_init_window;
+    int64_t remote_window;
+    int64_t announced_window;
+    int64_t announced_stream_total_over_incoming_window;
+    // BDP estimator stats.
+    int64_t bdp_accumulator;
+    int64_t bdp_estimate;
+    double bdp_bw_est;
+
+    std::string ToString() const;
+  };
+
+  Stats stats() const {
+    Stats stats;
+    stats.target_window = target_window();
+    stats.target_frame_size = target_frame_size();
+    stats.target_preferred_rx_crypto_frame_size =
+        target_preferred_rx_crypto_frame_size();
+    stats.acked_init_window = acked_init_window();
+    stats.queued_init_window = queued_init_window();
+    stats.sent_init_window = sent_init_window();
+    stats.remote_window = remote_window();
+    stats.announced_window = announced_window();
+    stats.announced_stream_total_over_incoming_window =
+        announced_stream_total_over_incoming_window();
+    stats.bdp_accumulator = bdp_estimator_.accumulator();
+    stats.bdp_estimate = bdp_estimator_.EstimateBdp();
+    stats.bdp_bw_est = bdp_estimator_.EstimateBandwidth();
+    return stats;
+  }
+
  private:
-  double TargetLogBdp();
-  double SmoothLogBdp(double value);
   double TargetInitialWindowSizeBasedOnMemoryPressureAndBdp() const;
-  static void UpdateSetting(grpc_chttp2_setting_id id, int64_t* desired_value,
+  static void UpdateSetting(absl::string_view name, int64_t* desired_value,
                             uint32_t new_desired_value,
                             FlowControlAction* action,
                             FlowControlAction& (FlowControlAction::*set)(
@@ -307,10 +356,6 @@ class TransportFlowControl final {
   // bdp estimation
   BdpEstimator bdp_estimator_;
 
-  // pid controller
-  PidController pid_controller_;
-  Timestamp last_pid_update_;
-
   int64_t remote_window_ = kDefaultWindow;
   int64_t target_initial_window_size_ = kDefaultWindow;
   int64_t target_frame_size_ = kDefaultFrameSize;
@@ -318,6 +363,7 @@ class TransportFlowControl final {
       kDefaultPreferredRxCryptoFrameSize;
   int64_t announced_window_ = kDefaultWindow;
   uint32_t acked_init_window_ = kDefaultWindow;
+  uint32_t sent_init_window_ = kDefaultWindow;
 };
 
 // Implementation of flow control that abides to HTTP/2 spec and attempts
@@ -378,12 +424,41 @@ class StreamFlowControl final {
 
   // returns an announce if we should send a stream update to our peer, else
   // returns zero
-  uint32_t MaybeSendUpdate();
+  uint32_t DesiredAnnounceSize() const;
+  // notify that we've actually sent a stream window update
+  // (should be DesiredAnnounceSize())
+  void SentUpdate(uint32_t announce);
+
+  // Older API: combines getting the DesiredAnnounceSize() with SentUpdate()
+  uint32_t MaybeSendUpdate() {
+    uint32_t n = DesiredAnnounceSize();
+    SentUpdate(n);
+    return n;
+  }
 
   int64_t remote_window_delta() const { return remote_window_delta_; }
   int64_t announced_window_delta() const { return announced_window_delta_; }
   int64_t min_progress_size() const { return min_progress_size_; }
 
+  // A snapshot of the flow control stats to export.
+  struct Stats {
+    int64_t min_progress_size;
+    int64_t remote_window_delta;
+    int64_t announced_window_delta;
+    absl::optional<int64_t> pending_size;
+
+    std::string ToString() const;
+  };
+
+  Stats stats() const {
+    Stats stats;
+    stats.min_progress_size = min_progress_size();
+    stats.remote_window_delta = remote_window_delta();
+    stats.announced_window_delta = announced_window_delta();
+    stats.pending_size = pending_size_;
+    return stats;
+  }
+
  private:
   TransportFlowControl* const tfc_;
   int64_t min_progress_size_ = 0;
@@ -392,7 +467,6 @@ class StreamFlowControl final {
   absl::optional<int64_t> pending_size_;
 
   FlowControlAction UpdateAction(FlowControlAction action);
-  int64_t DesiredAnnounceSize() const;
 };
 
 class TestOnlyTransportTargetWindowEstimatesMocker {
