@@ -14,8 +14,8 @@
 // limitations under the License.
 //
 
-#ifndef GRPC_CORE_EXT_XDS_XDS_CLIENT_GRPC_H
-#define GRPC_CORE_EXT_XDS_XDS_CLIENT_GRPC_H
+#ifndef GRPC_SRC_CORE_EXT_XDS_XDS_CLIENT_GRPC_H
+#define GRPC_SRC_CORE_EXT_XDS_XDS_CLIENT_GRPC_H
 
 #include <grpc/support/port_platform.h>
 
@@ -29,11 +29,13 @@
 #include "src/core/ext/xds/certificate_provider_store.h"
 #include "src/core/ext/xds/xds_bootstrap_grpc.h"
 #include "src/core/ext/xds/xds_client.h"
+#include "src/core/ext/xds/xds_transport.h"
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/gpr/useful.h"
 #include "src/core/lib/gprpp/orphanable.h"
 #include "src/core/lib/gprpp/ref_counted_ptr.h"
 #include "src/core/lib/iomgr/iomgr_fwd.h"
+#include "src/core/resolver/endpoint_addresses.h"
 
 namespace grpc_core {
 
@@ -43,14 +45,29 @@ class GrpcXdsClient : public XdsClient {
   static absl::StatusOr<RefCountedPtr<GrpcXdsClient>> GetOrCreate(
       const ChannelArgs& args, const char* reason);
 
+  // Builds ClientStatusResponse containing all resources from all XdsClients
+  static grpc_slice DumpAllClientConfigs();
+
   // Do not instantiate directly -- use GetOrCreate() instead.
+  // TODO(roth): The transport factory is injectable here to support
+  // tests that want to use a fake transport factory with code that
+  // expects a GrpcXdsClient instead of an XdsClient, typically because
+  // it needs to call the interested_parties() method.  Once we
+  // finish the EventEngine migration and remove the interested_parties()
+  // method, consider instead changing callers to an approach where the
+  // production code uses XdsClient instead of GrpcXdsClient, and then
+  // passing in a fake XdsClient impl in the tests.  Note that this will
+  // work for callers that use interested_parties() but not for callers
+  // that also use certificate_provider_store(), but we should consider
+  // alternatives for that case as well.
   GrpcXdsClient(std::unique_ptr<GrpcXdsBootstrap> bootstrap,
-                const ChannelArgs& args);
+                const ChannelArgs& args,
+                OrphanablePtr<XdsTransportFactory> transport_factory);
   ~GrpcXdsClient() override;
 
   // Helpers for encoding the XdsClient object in channel args.
   static absl::string_view ChannelArgName() {
-    return "grpc.internal.xds_client";
+    return GRPC_ARG_NO_SUBCHANNEL_PREFIX "xds_client";
   }
   static int ChannelArgsCompare(const XdsClient* a, const XdsClient* b) {
     return QsortCompare(a, b);
@@ -76,4 +93,4 @@ void SetXdsFallbackBootstrapConfig(const char* config);
 
 }  // namespace grpc_core
 
-#endif  // GRPC_CORE_EXT_XDS_XDS_CLIENT_GRPC_H
+#endif  // GRPC_SRC_CORE_EXT_XDS_XDS_CLIENT_GRPC_H
