@@ -22,7 +22,6 @@
 
 #include <string.h>
 
-#include <algorithm>
 #include <utility>
 
 #include "absl/strings/cord.h"
@@ -34,9 +33,8 @@
 #include "absl/time/clock.h"
 #include "google/protobuf/any.upb.h"
 #include "google/rpc/status.upb.h"
-#include "upb/arena.h"
-#include "upb/upb.h"
-#include "upb/upb.hpp"
+#include "upb/base/string_view.h"
+#include "upb/mem/arena.hpp"
 
 #include <grpc/support/log.h>
 
