@@ -12,8 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#ifndef GRPC_CORE_LIB_IOMGR_RESOLVED_ADDRESS_H
-#define GRPC_CORE_LIB_IOMGR_RESOLVED_ADDRESS_H
+#ifndef GRPC_SRC_CORE_LIB_IOMGR_RESOLVED_ADDRESS_H
+#define GRPC_SRC_CORE_LIB_IOMGR_RESOLVED_ADDRESS_H
 
 #include <grpc/support/port_platform.h>
 
@@ -36,4 +36,4 @@ struct grpc_resolved_address {
   socklen_t len;
 };
 
-#endif  // GRPC_CORE_LIB_IOMGR_RESOLVED_ADDRESS_H
+#endif  // GRPC_SRC_CORE_LIB_IOMGR_RESOLVED_ADDRESS_H
