@@ -16,15 +16,24 @@
 //
 //
 
-#ifndef GRPC_CORE_LIB_JSON_JSON_UTIL_H
-#define GRPC_CORE_LIB_JSON_JSON_UTIL_H
+#ifndef GRPC_SRC_CORE_LIB_JSON_JSON_UTIL_H
+#define GRPC_SRC_CORE_LIB_JSON_JSON_UTIL_H
 
 #include <grpc/support/port_platform.h>
 
+#include <algorithm>
+#include <map>
+#include <string>
+#include <type_traits>
+#include <utility>
+#include <vector>
+
 #include "absl/strings/numbers.h"
 #include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
 
-#include "src/core/lib/iomgr/exec_ctx.h"
+#include "src/core/lib/gprpp/time.h"
+#include "src/core/lib/iomgr/error.h"
 #include "src/core/lib/json/json.h"
 
 namespace grpc_core {
@@ -45,13 +54,14 @@ bool ExtractJsonNumber(const Json& json, absl::string_view field_name,
                        NumericType* output,
                        std::vector<grpc_error_handle>* error_list) {
   static_assert(std::is_integral<NumericType>::value, "Integral required");
-  if (json.type() != Json::Type::NUMBER) {
-    error_list->push_back(GRPC_ERROR_CREATE_FROM_CPP_STRING(
-        absl::StrCat("field:", field_name, " error:type should be NUMBER")));
+  if (json.type() != Json::Type::kNumber &&
+      json.type() != Json::Type::kString) {
+    error_list->push_back(GRPC_ERROR_CREATE(absl::StrCat(
+        "field:", field_name, " error:type should be NUMBER or STRING")));
     return false;
   }
-  if (!absl::SimpleAtoi(json.string_value(), output)) {
-    error_list->push_back(GRPC_ERROR_CREATE_FROM_CPP_STRING(
+  if (!absl::SimpleAtoi(json.string(), output)) {
+    error_list->push_back(GRPC_ERROR_CREATE(
         absl::StrCat("field:", field_name, " error:failed to parse.")));
     return false;
   }
@@ -66,13 +76,13 @@ template <typename OutputType>
 bool ExtractJsonString(const Json& json, absl::string_view field_name,
                        OutputType* output,
                        std::vector<grpc_error_handle>* error_list) {
-  if (json.type() != Json::Type::STRING) {
+  if (json.type() != Json::Type::kString) {
     *output = "";
-    error_list->push_back(GRPC_ERROR_CREATE_FROM_CPP_STRING(
+    error_list->push_back(GRPC_ERROR_CREATE(
         absl::StrCat("field:", field_name, " error:type should be STRING")));
     return false;
   }
-  *output = json.string_value();
+  *output = json.string();
   return true;
 }
 
@@ -133,7 +143,7 @@ bool ParseJsonObjectField(const Json::Object& object,
   auto it = object.find(std::string(field_name));
   if (it == object.end()) {
     if (required) {
-      error_list->push_back(GRPC_ERROR_CREATE_FROM_CPP_STRING(
+      error_list->push_back(GRPC_ERROR_CREATE(
           absl::StrCat("field:", field_name, " error:does not exist.")));
     }
     return false;
@@ -151,4 +161,4 @@ bool ParseJsonObjectFieldAsDuration(const Json::Object& object,
 
 }  // namespace grpc_core
 
-#endif  // GRPC_CORE_LIB_JSON_JSON_UTIL_H
+#endif  // GRPC_SRC_CORE_LIB_JSON_JSON_UTIL_H
