@@ -1,48 +1,50 @@
-/*
- *
- * Copyright 2015 gRPC authors.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- *
- */
+//
+//
+// Copyright 2015 gRPC authors.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
+//
 
 #include <grpc/support/port_platform.h>
 
+#include <stdint.h>
 #include <stdlib.h>
-#include <string.h>
 
-#include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
+#include <map>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include "absl/strings/string_view.h"
 
 #include "src/core/lib/json/json.h"
 
-#include "src/core/lib/gprpp/string_view.h"
-
 namespace grpc_core {
 
 namespace {
 
-/* The idea of the writer is basically symmetrical of the reader. While the
- * reader emits various calls to your code, the writer takes basically the
- * same calls and emit json out of it. It doesn't try to make any check on
- * the order of the calls you do on it. Meaning you can theorically force
- * it to generate invalid json.
- *
- * Also, unlike the reader, the writer expects UTF-8 encoded input strings.
- * These strings will be UTF-8 validated, and any invalid character will
- * cut the conversion short, before any invalid UTF-8 sequence, thus forming
- * a valid UTF-8 string overall.
- */
+// The idea of the writer is basically symmetrical of the reader. While the
+// reader emits various calls to your code, the writer takes basically the
+// same calls and emit json out of it. It doesn't try to make any check on
+// the order of the calls you do on it. Meaning you can theorically force
+// it to generate invalid json.
+//
+// Also, unlike the reader, the writer expects UTF-8 encoded input strings.
+// These strings will be UTF-8 validated, and any invalid character will
+// cut the conversion short, before any invalid UTF-8 sequence, thus forming
+// a valid UTF-8 string overall.
+//
 class JsonWriter {
  public:
   static std::string Dump(const Json& value, int indent);
@@ -52,7 +54,7 @@ class JsonWriter {
 
   void OutputCheck(size_t needed);
   void OutputChar(char c);
-  void OutputString(const StringView str);
+  void OutputString(const absl::string_view str);
   void OutputIndent();
   void ValueEnd();
   void EscapeUtf16(uint16_t utf16);
@@ -74,15 +76,15 @@ class JsonWriter {
   std::string output_;
 };
 
-/* This function checks if there's enough space left in the output buffer,
- * and will enlarge it if necessary. We're only allocating chunks of 256
- * bytes at a time (or multiples thereof).
- */
+// This function checks if there's enough space left in the output buffer,
+// and will enlarge it if necessary. We're only allocating chunks of 256
+// bytes at a time (or multiples thereof).
+//
 void JsonWriter::OutputCheck(size_t needed) {
   size_t free_space = output_.capacity() - output_.size();
   if (free_space >= needed) return;
   needed -= free_space;
-  /* Round up by 256 bytes. */
+  // Round up by 256 bytes.
   needed = (needed + 0xff) & ~0xffU;
   output_.reserve(output_.capacity() + needed);
 }
@@ -92,7 +94,7 @@ void JsonWriter::OutputChar(char c) {
   output_.push_back(c);
 }
 
-void JsonWriter::OutputString(const StringView str) {
+void JsonWriter::OutputString(const absl::string_view str) {
   OutputCheck(str.size());
   output_.append(str.data(), str.size());
 }
@@ -110,11 +112,12 @@ void JsonWriter::OutputIndent() {
     return;
   }
   while (spaces >= (sizeof(spacesstr) - 1)) {
-    OutputString(StringView(spacesstr, sizeof(spacesstr) - 1));
+    OutputString(absl::string_view(spacesstr, sizeof(spacesstr) - 1));
     spaces -= static_cast<unsigned>(sizeof(spacesstr) - 1);
   }
   if (spaces == 0) return;
-  OutputString(StringView(spacesstr + sizeof(spacesstr) - 1 - spaces, spaces));
+  OutputString(
+      absl::string_view(spacesstr + sizeof(spacesstr) - 1 - spaces, spaces));
 }
 
 void JsonWriter::ValueEnd() {
@@ -131,38 +134,36 @@ void JsonWriter::ValueEnd() {
 
 void JsonWriter::EscapeUtf16(uint16_t utf16) {
   static const char hex[] = "0123456789abcdef";
-  OutputString(StringView("\\u", 2));
+  OutputString(absl::string_view("\\u", 2));
   OutputChar(hex[(utf16 >> 12) & 0x0f]);
   OutputChar(hex[(utf16 >> 8) & 0x0f]);
   OutputChar(hex[(utf16 >> 4) & 0x0f]);
-  OutputChar(hex[(utf16)&0x0f]);
+  OutputChar(hex[(utf16) & 0x0f]);
 }
 
 void JsonWriter::EscapeString(const std::string& string) {
   OutputChar('"');
   for (size_t idx = 0; idx < string.size(); ++idx) {
     uint8_t c = static_cast<uint8_t>(string[idx]);
-    if (c == 0) {
-      break;
-    } else if (c >= 32 && c <= 126) {
+    if (c >= 32 && c <= 126) {
       if (c == '\\' || c == '"') OutputChar('\\');
       OutputChar(static_cast<char>(c));
     } else if (c < 32 || c == 127) {
       switch (c) {
         case '\b':
-          OutputString(StringView("\\b", 2));
+          OutputString(absl::string_view("\\b", 2));
           break;
         case '\f':
-          OutputString(StringView("\\f", 2));
+          OutputString(absl::string_view("\\f", 2));
           break;
         case '\n':
-          OutputString(StringView("\\n", 2));
+          OutputString(absl::string_view("\\n", 2));
           break;
         case '\r':
-          OutputString(StringView("\\r", 2));
+          OutputString(absl::string_view("\\r", 2));
           break;
         case '\t':
-          OutputString(StringView("\\t", 2));
+          OutputString(absl::string_view("\\t", 2));
           break;
         default:
           EscapeUtf16(c);
@@ -188,13 +189,13 @@ void JsonWriter::EscapeString(const std::string& string) {
       for (i = 0; i < extra; i++) {
         utf32 <<= 6;
         ++idx;
-        /* Breaks out and bail if we hit the end of the string. */
+        // Breaks out and bail if we hit the end of the string.
         if (idx == string.size()) {
           valid = 0;
           break;
         }
         c = static_cast<uint8_t>(string[idx]);
-        /* Breaks out and bail on any invalid UTF-8 sequence, including \0. */
+        // Breaks out and bail on any invalid UTF-8 sequence, including \0.
         if ((c & 0xc0) != 0x80) {
           valid = 0;
           break;
@@ -202,29 +203,30 @@ void JsonWriter::EscapeString(const std::string& string) {
         utf32 |= c & 0x3f;
       }
       if (!valid) break;
-      /* The range 0xd800 - 0xdfff is reserved by the surrogates ad vitam.
-       * Any other range is technically reserved for future usage, so if we
-       * don't want the software to break in the future, we have to allow
-       * anything else. The first non-unicode character is 0x110000. */
-      if (((utf32 >= 0xd800) && (utf32 <= 0xdfff)) || (utf32 >= 0x110000))
+      // The range 0xd800 - 0xdfff is reserved by the surrogates ad vitam.
+      // Any other range is technically reserved for future usage, so if we
+      // don't want the software to break in the future, we have to allow
+      // anything else. The first non-unicode character is 0x110000.
+      if (((utf32 >= 0xd800) && (utf32 <= 0xdfff)) || (utf32 >= 0x110000)) {
         break;
+      }
       if (utf32 >= 0x10000) {
-        /* If utf32 contains a character that is above 0xffff, it needs to be
-         * broken down into a utf-16 surrogate pair. A surrogate pair is first
-         * a high surrogate, followed by a low surrogate. Each surrogate holds
-         * 10 bits of usable data, thus allowing a total of 20 bits of data.
-         * The high surrogate marker is 0xd800, while the low surrogate marker
-         * is 0xdc00. The low 10 bits of each will be the usable data.
-         *
-         * After re-combining the 20 bits of data, one has to add 0x10000 to
-         * the resulting value, in order to obtain the original character.
-         * This is obviously because the range 0x0000 - 0xffff can be written
-         * without any special trick.
-         *
-         * Since 0x10ffff is the highest allowed character, we're working in
-         * the range 0x00000 - 0xfffff after we decrement it by 0x10000.
-         * That range is exactly 20 bits.
-         */
+        // If utf32 contains a character that is above 0xffff, it needs to be
+        // broken down into a utf-16 surrogate pair. A surrogate pair is first
+        // a high surrogate, followed by a low surrogate. Each surrogate holds
+        // 10 bits of usable data, thus allowing a total of 20 bits of data.
+        // The high surrogate marker is 0xd800, while the low surrogate marker
+        // is 0xdc00. The low 10 bits of each will be the usable data.
+        //
+        // After re-combining the 20 bits of data, one has to add 0x10000 to
+        // the resulting value, in order to obtain the original character.
+        // This is obviously because the range 0x0000 - 0xffff can be written
+        // without any special trick.
+        //
+        // Since 0x10ffff is the highest allowed character, we're working in
+        // the range 0x00000 - 0xfffff after we decrement it by 0x10000.
+        // That range is exactly 20 bits.
+        //
         utf32 -= 0x10000;
         EscapeUtf16(static_cast<uint16_t>(0xd800 | (utf32 >> 10)));
         EscapeUtf16(static_cast<uint16_t>(0xdc00 | (utf32 & 0x3ff)));
@@ -239,7 +241,7 @@ void JsonWriter::EscapeString(const std::string& string) {
 void JsonWriter::ContainerBegins(Json::Type type) {
   if (!got_key_) ValueEnd();
   OutputIndent();
-  OutputChar(type == Json::Type::OBJECT ? '{' : '[');
+  OutputChar(type == Json::Type::kObject ? '{' : '[');
   container_empty_ = true;
   got_key_ = false;
   depth_++;
@@ -249,7 +251,7 @@ void JsonWriter::ContainerEnds(Json::Type type) {
   if (indent_ && !container_empty_) OutputChar('\n');
   depth_--;
   if (!container_empty_) OutputIndent();
-  OutputChar(type == Json::Type::OBJECT ? '}' : ']');
+  OutputChar(type == Json::Type::kObject ? '}' : ']');
   container_empty_ = false;
   got_key_ = false;
 }
@@ -277,43 +279,44 @@ void JsonWriter::ValueString(const std::string& string) {
 }
 
 void JsonWriter::DumpObject(const Json::Object& object) {
-  ContainerBegins(Json::Type::OBJECT);
+  ContainerBegins(Json::Type::kObject);
   for (const auto& p : object) {
-    ObjectKey(p.first.data());
+    ObjectKey(p.first);
     DumpValue(p.second);
   }
-  ContainerEnds(Json::Type::OBJECT);
+  ContainerEnds(Json::Type::kObject);
 }
 
 void JsonWriter::DumpArray(const Json::Array& array) {
-  ContainerBegins(Json::Type::ARRAY);
+  ContainerBegins(Json::Type::kArray);
   for (const auto& v : array) {
     DumpValue(v);
   }
-  ContainerEnds(Json::Type::ARRAY);
+  ContainerEnds(Json::Type::kArray);
 }
 
 void JsonWriter::DumpValue(const Json& value) {
   switch (value.type()) {
-    case Json::Type::OBJECT:
-      DumpObject(value.object_value());
+    case Json::Type::kObject:
+      DumpObject(value.object());
       break;
-    case Json::Type::ARRAY:
-      DumpArray(value.array_value());
+    case Json::Type::kArray:
+      DumpArray(value.array());
       break;
-    case Json::Type::STRING:
-      ValueString(value.string_value());
+    case Json::Type::kString:
+      ValueString(value.string());
       break;
-    case Json::Type::NUMBER:
-      ValueRaw(value.string_value());
+    case Json::Type::kNumber:
+      ValueRaw(value.string());
       break;
-    case Json::Type::JSON_TRUE:
-      ValueRaw(std::string("true", 4));
+    case Json::Type::kBoolean:
+      if (value.boolean()) {
+        ValueRaw(std::string("true", 4));
+      } else {
+        ValueRaw(std::string("false", 5));
+      }
       break;
-    case Json::Type::JSON_FALSE:
-      ValueRaw(std::string("false", 5));
-      break;
-    case Json::Type::JSON_NULL:
+    case Json::Type::kNull:
       ValueRaw(std::string("null", 4));
       break;
     default:
@@ -329,8 +332,8 @@ std::string JsonWriter::Dump(const Json& value, int indent) {
 
 }  // namespace
 
-std::string Json::Dump(int indent) const {
-  return JsonWriter::Dump(*this, indent);
+std::string JsonDump(const Json& json, int indent) {
+  return JsonWriter::Dump(json, indent);
 }
 
 }  // namespace grpc_core
