@@ -12,8 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#ifndef GRPC_CORE_LIB_MATCHERS_MATCHERS_H
-#define GRPC_CORE_LIB_MATCHERS_MATCHERS_H
+#ifndef GRPC_SRC_CORE_LIB_MATCHERS_MATCHERS_H
+#define GRPC_SRC_CORE_LIB_MATCHERS_MATCHERS_H
 
 #include <grpc/support/port_platform.h>
 
@@ -113,7 +113,8 @@ class HeaderMatcher {
                                               int64_t range_start = 0,
                                               int64_t range_end = 0,
                                               bool present_match = false,
-                                              bool invert_match = false);
+                                              bool invert_match = false,
+                                              bool case_sensitive = true);
 
   HeaderMatcher() = default;
   HeaderMatcher(const HeaderMatcher& other);
@@ -159,4 +160,4 @@ class HeaderMatcher {
 
 }  // namespace grpc_core
 
-#endif  // GRPC_CORE_LIB_MATCHERS_MATCHERS_H
+#endif  // GRPC_SRC_CORE_LIB_MATCHERS_MATCHERS_H
