@@ -18,6 +18,12 @@
 
 #include <stddef.h>
 
+#include <vector>
+
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_format.h"
+#include "absl/strings/str_join.h"
+
 #include "src/core/lib/gprpp/atomic_utils.h"
 
 namespace grpc_core {
@@ -25,13 +31,17 @@ namespace grpc_core {
 ///////////////////////////////////////////////////////////////////////////////
 // GLOBALS
 
-GPR_THREAD_LOCAL(Activity*) Activity::g_current_activity_{nullptr};
+thread_local Activity* Activity::g_current_activity_{nullptr};
 
 namespace promise_detail {
 
 ///////////////////////////////////////////////////////////////////////////////
 // HELPER TYPES
 
+std::string Unwakeable::ActivityDebugTag(WakeupMask) const {
+  return "<unknown>";
+}
+
 // Weak handle to an Activity.
 // Handle can persist while Activity goes away.
 class FreestandingActivity::Handle final : public Wakeable {
@@ -52,7 +62,7 @@ class FreestandingActivity::Handle final : public Wakeable {
 
   // Activity needs to wake up (if it still exists!) - wake it up, and drop the
   // ref that was kept for this handle.
-  void Wakeup() override ABSL_LOCKS_EXCLUDED(mu_) {
+  void Wakeup(WakeupMask) override ABSL_LOCKS_EXCLUDED(mu_) {
     mu_.Lock();
     // Note that activity refcount can drop to zero, but we could win the lock
     // against DropActivity, so we need to only increase activities refcount if
@@ -62,7 +72,7 @@ class FreestandingActivity::Handle final : public Wakeable {
       mu_.Unlock();
       // Activity still exists and we have a reference: wake it up, which will
       // drop the ref.
-      activity->Wakeup();
+      activity->Wakeup(0);
     } else {
       // Could not get the activity - it's either gone or going. No need to wake
       // it up!
@@ -72,7 +82,32 @@ class FreestandingActivity::Handle final : public Wakeable {
     Unref();
   }
 
-  void Drop() override { Unref(); }
+  void WakeupAsync(WakeupMask) override ABSL_LOCKS_EXCLUDED(mu_) {
+    mu_.Lock();
+    // Note that activity refcount can drop to zero, but we could win the lock
+    // against DropActivity, so we need to only increase activities refcount if
+    // it is non-zero.
+    if (activity_ && activity_->RefIfNonzero()) {
+      FreestandingActivity* activity = activity_;
+      mu_.Unlock();
+      // Activity still exists and we have a reference: wake it up, which will
+      // drop the ref.
+      activity->WakeupAsync(0);
+    } else {
+      // Could not get the activity - it's either gone or going. No need to wake
+      // it up!
+      mu_.Unlock();
+    }
+    // Drop the ref to the handle (we have one ref = one wakeup semantics).
+    Unref();
+  }
+
+  void Drop(WakeupMask) override { Unref(); }
+
+  std::string ActivityDebugTag(WakeupMask) const override {
+    MutexLock lock(&mu_);
+    return activity_ == nullptr ? "<unknown>" : activity_->DebugTag();
+  }
 
  private:
   // Unref the Handle (not the activity).
@@ -85,7 +120,7 @@ class FreestandingActivity::Handle final : public Wakeable {
   // Two initial refs: one for the waiter that caused instantiation, one for the
   // activity.
   std::atomic<size_t> refs_{2};
-  Mutex mu_ ABSL_ACQUIRED_AFTER(activity_->mu_);
+  mutable Mutex mu_ ABSL_ACQUIRED_AFTER(activity_->mu_);
   FreestandingActivity* activity_ ABSL_GUARDED_BY(mu_);
 };
 
@@ -113,8 +148,24 @@ void FreestandingActivity::DropHandle() {
 
 Waker FreestandingActivity::MakeNonOwningWaker() {
   mu_.AssertHeld();
-  return Waker(RefHandle());
+  return Waker(RefHandle(), 0);
 }
 
 }  // namespace promise_detail
+
+std::string Activity::DebugTag() const {
+  return absl::StrFormat("ACTIVITY[%p]", this);
+}
+
+///////////////////////////////////////////////////////////////////////////////
+// INTRA ACTIVITY WAKER IMPLEMENTATION
+
+std::string IntraActivityWaiter::DebugString() const {
+  std::vector<int> bits;
+  for (size_t i = 0; i < 8 * sizeof(WakeupMask); i++) {
+    if (wakeups_ & (1 << i)) bits.push_back(i);
+  }
+  return absl::StrCat("{", absl::StrJoin(bits, ","), "}");
+}
+
 }  // namespace grpc_core
