@@ -12,12 +12,16 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#ifndef GRPC_CORE_LIB_RESOURCE_QUOTA_API_H
-#define GRPC_CORE_LIB_RESOURCE_QUOTA_API_H
+#ifndef GRPC_SRC_CORE_LIB_RESOURCE_QUOTA_API_H
+#define GRPC_SRC_CORE_LIB_RESOURCE_QUOTA_API_H
 
 #include <grpc/support/port_platform.h>
 
-#include "src/core/lib/channel/channel_args.h"
+#include <stddef.h>
+
+#include <grpc/event_engine/endpoint_config.h>
+#include <grpc/grpc.h>
+
 #include "src/core/lib/config/core_configuration.h"
 #include "src/core/lib/resource_quota/resource_quota.h"
 
@@ -27,14 +31,19 @@ namespace grpc_core {
 
 // TODO(ctiller): This is a hack. We need to do real accounting instead of
 // hard coding.
-constexpr size_t kResourceQuotaChannelSize = 50 * 1024;
+constexpr size_t kResourceQuotaChannelSize = 34 * 1024;
 
 // Retrieve the resource quota from the channel args.
 // UB if not set.
 ResourceQuotaRefPtr ResourceQuotaFromChannelArgs(const grpc_channel_args* args);
 
+// Retrieve the resource quota from the EndpointConfig.
+// Returns nullptr if not set.
+ResourceQuotaRefPtr ResourceQuotaFromEndpointConfig(
+    const grpc_event_engine::experimental::EndpointConfig& config);
+
 void RegisterResourceQuota(CoreConfiguration::Builder* builder);
 
 }  // namespace grpc_core
 
-#endif  // GRPC_CORE_LIB_RESOURCE_QUOTA_API_H
+#endif  // GRPC_SRC_CORE_LIB_RESOURCE_QUOTA_API_H
