@@ -26,7 +26,7 @@
 #include "absl/container/flat_hash_set.h"
 #include "envoy/config/rbac/v3/rbac.upb.h"
 #include "google/api/expr/v1alpha1/syntax.upb.h"
-#include "upb/upb.hpp"
+#include "upb/mem/arena.hpp"
 
 #include "src/core/lib/security/authorization/evaluate_args.h"
 #include "src/core/lib/security/authorization/mock_cel/activation.h"
