@@ -12,8 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#ifndef GRPC_CORE_LIB_SLICE_SLICE_H
-#define GRPC_CORE_LIB_SLICE_SLICE_H
+#ifndef GRPC_SRC_CORE_LIB_SLICE_SLICE_H
+#define GRPC_SRC_CORE_LIB_SLICE_SLICE_H
 
 #include <grpc/support/port_platform.h>
 
@@ -31,6 +31,7 @@
 #include <grpc/support/log.h>
 
 #include "src/core/lib/gpr/string.h"
+#include "src/core/lib/gprpp/debug_location.h"
 #include "src/core/lib/slice/slice_internal.h"
 #include "src/core/lib/slice/slice_refcount.h"
 
@@ -47,23 +48,24 @@
 //   MutableSlice - provides a guarantee of unique ownership, meaning the
 //                  underlying data can be mutated safely.
 
-// This slice implementation is an extension of the event engine Slice
+// This slice implementation is an extension of the EventEngine Slice
 // implementation defined in <grpc/event_engine/slice.h>. Changes to this
-// implementation might warrant changes to the public event engine Slice
+// implementation might warrant changes to the public EventEngine Slice
 // type as well.
 
 namespace grpc_core {
 
-inline const grpc_slice& CSliceRef(const grpc_slice& slice) {
+inline const grpc_slice& CSliceRef(const grpc_slice& slice,
+                                   DebugLocation loc = {}) {
   if (reinterpret_cast<uintptr_t>(slice.refcount) > 1) {
-    slice.refcount->Ref();
+    slice.refcount->Ref(loc);
   }
   return slice;
 }
 
-inline void CSliceUnref(const grpc_slice& slice) {
+inline void CSliceUnref(const grpc_slice& slice, DebugLocation loc = {}) {
   if (reinterpret_cast<uintptr_t>(slice.refcount) > 1) {
-    slice.refcount->Unref();
+    slice.refcount->Unref(loc);
   }
 }
 
@@ -289,6 +291,12 @@ class GPR_MSVC_EMPTY_BASE_CLASS_WORKAROUND MutableSlice
     return MutableSlice(grpc_slice_sub_no_ref(TakeCSlice(), pos, pos + n));
   }
 
+  // Split this slice in two, returning the first n bytes and leaving the
+  // remainder.
+  MutableSlice TakeFirst(size_t n) {
+    return MutableSlice(NoCheck{}, grpc_slice_split_head(c_slice_ptr(), n));
+  }
+
   // Iterator access to the underlying bytes
   uint8_t* begin() { return mutable_data(); }
   uint8_t* end() { return mutable_data() + size(); }
@@ -296,6 +304,13 @@ class GPR_MSVC_EMPTY_BASE_CLASS_WORKAROUND MutableSlice
 
   // Array access
   uint8_t& operator[](size_t i) { return mutable_data()[i]; }
+
+  using slice_detail::BaseSlice::c_slice_ptr;
+
+ private:
+  struct NoCheck {};
+  MutableSlice(NoCheck, const grpc_slice& slice)
+      : slice_detail::BaseSlice(slice) {}
 };
 
 class GPR_MSVC_EMPTY_BASE_CLASS_WORKAROUND Slice
@@ -338,6 +353,22 @@ class GPR_MSVC_EMPTY_BASE_CLASS_WORKAROUND Slice
     return Slice(TakeCSlice());
   }
 
+  // As per TakeOwned, but if the slice is refcounted and there are other refs
+  // then it will copy instead of ref-counting, to ensure the returned slice is
+  // not shared.
+  Slice TakeUniquelyOwned() {
+    if (c_slice().refcount == nullptr) {
+      return Slice(c_slice());
+    }
+    if (c_slice().refcount == grpc_slice_refcount::NoopRefcount()) {
+      return Slice(grpc_slice_copy(c_slice()));
+    }
+    if (c_slice().refcount->IsUnique()) {
+      return Slice(TakeCSlice());
+    }
+    return Slice(grpc_slice_copy(c_slice()));
+  }
+
   // AsOwned returns an owned slice but does not mutate the current slice,
   // meaning that it may add a reference to the underlying slice.
   Slice AsOwned() const {
@@ -391,10 +422,11 @@ class GPR_MSVC_EMPTY_BASE_CLASS_WORKAROUND Slice
   Slice Copy() const { return Slice(grpc_slice_copy(c_slice())); }
 
   static Slice FromRefcountAndBytes(grpc_slice_refcount* r,
-                                    const uint8_t* begin, const uint8_t* end) {
+                                    const uint8_t* begin, const uint8_t* end,
+                                    DebugLocation location = {}) {
     grpc_slice out;
     out.refcount = r;
-    if (r != grpc_slice_refcount::NoopRefcount()) r->Ref();
+    if (r != grpc_slice_refcount::NoopRefcount()) r->Ref(location);
     out.data.refcounted.bytes = const_cast<uint8_t*>(begin);
     out.data.refcounted.length = end - begin;
     return Slice(out);
@@ -418,18 +450,8 @@ template <>
 struct SliceCastable<grpc_core::Slice, Slice> {};
 template <>
 struct SliceCastable<Slice, grpc_core::Slice> {};
-
-template <>
-struct SliceCastable<grpc_core::MutableSlice, grpc_slice> {};
-template <>
-struct SliceCastable<grpc_slice, grpc_core::MutableSlice> {};
-
-template <>
-struct SliceCastable<grpc_core::MutableSlice, grpc_core::Slice> {};
-template <>
-struct SliceCastable<grpc_core::Slice, grpc_core::MutableSlice> {};
 }  // namespace internal
 }  // namespace experimental
 }  // namespace grpc_event_engine
 
-#endif  // GRPC_CORE_LIB_SLICE_SLICE_H
+#endif  // GRPC_SRC_CORE_LIB_SLICE_SLICE_H
