@@ -12,8 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#ifndef GRPC_CORE_LIB_SURFACE_INIT_INTERNALLY_H
-#define GRPC_CORE_LIB_SURFACE_INIT_INTERNALLY_H
+#ifndef GRPC_SRC_CORE_LIB_SURFACE_INIT_INTERNALLY_H
+#define GRPC_SRC_CORE_LIB_SURFACE_INIT_INTERNALLY_H
 
 namespace grpc_core {
 
@@ -34,4 +34,4 @@ class KeepsGrpcInitialized {
 
 }  // namespace grpc_core
 
-#endif  // GRPC_CORE_LIB_SURFACE_INIT_INTERNALLY_H
+#endif  // GRPC_SRC_CORE_LIB_SURFACE_INIT_INTERNALLY_H
